/* $Id: diskdumpcmn.c,v 1.13 2005/06/28 14:12:27 tuchida Exp $ */

#include <glib.h>
#include <stdio.h>
#include <stdlib.h>
#include <stdbool.h>
#include <string.h>
#include <time.h>
#include <sys/stat.h>
#include <termio.h>
#include "version.h"
#include "diskdumpcmn.h"

#define Stderr(x, ...)	fprintf(stderr, x, ## __VA_ARGS__)

static const u64 failed = (u64)-1;

#define DEFAULT_WINSZ	80

struct option {
	char	*kind;	/* character used as an option */
	char	*cmnt;	/* description to an option */
};

static void set_opt(struct option *opt, char *kind, char *comment)
{
	opt->kind = kind;
	opt->cmnt = comment;
}

static void print_opt(struct option *opt, int n)
{
	int i;

	for (i = 0; i < n; i++) {
		if (opt[i].kind != NULL) {
			Stderr("\t%s:   %s\n", opt[i].kind, opt[i].cmnt);
		}
	}
}

void usage(const char *cmd)
{
	int n = 0;
	const char *opts;
	struct option opt[4] = {
		{NULL, NULL},
		{NULL, NULL},
		{NULL, NULL},
		{NULL, NULL}
	};
	int ret;

	/*
	 * -v: some verbose outputs.
	 * -V: version information.
	 * Both of them are common options for each command.
	 */
	if (strcmp(cmd, "diskdumpfmt") == 0) {
		opts = "[-v] [[-a] [-p] [-f]]|-c]";
		set_opt(&opt[n++], "-a",
				"format all blocks of the dump partition");
		set_opt(&opt[n++], "-c",
				"check the status of the dump partition");
		set_opt(&opt[n++], "-f",
				"format the dump partition forcibly");
		set_opt(&opt[n++], "-p",
				"display progress for formatting");
		ret = 255;
	} else if (strcmp(cmd, "savecore") == 0) {
		opts = "[-D] [-f] [-p] [-v]";
		set_opt(&opt[n++], "-D",
				"delete dump from the dump partition");
		set_opt(&opt[n++], "-f",
		"create a dump file forcibly even if dump directory exists");
		set_opt(&opt[n++], "-p",
				"display progress for creating a dump file");
		ret = 1;
	} else if (strcmp(cmd, "diskdumpctl") == 0) {
		opts = "[-u] [-v]";
		set_opt(&opt[n++], "-u",
				"unregister the device");
		ret = 1;
	} else {
		Stderr("Error: command '%s' does not match.\n", cmd);
		exit(-1);
	}

	Stderr("Usage: %s %s device\n", cmd, opts);
	Stderr("       %s -V\n", cmd);

	Stderr("Available options:\n");
	print_opt(opt, n);	/* shown as "\toption:   description" */
	Stderr("\t-v:   provide some verbose outputs\n");
	Stderr("\t-V:   show %s version and exit\n", cmd);

	exit(ret);
}

void version(const char *cmd)
{
	Stderr("%s version %s\n", cmd, CMD_VERSION);

	exit(0);
}

int convert_version_number(struct version_form *ver, const char *v_str)
{
	/* Initialize the version number as "0.0.0". */
	ver->major = 0;
	ver->minor = 0;
	ver->micro = 0;

	return sscanf(v_str, "%d.%d.%d", &ver->major, &ver->minor, &ver->micro);
}

/*
 * It gets some format parameters, sample_rate, block_order,
 * total_blocks and so on, from "/proc/diskdump".
 */
u64 scan_param_proc(const char *param_name)
{
	FILE *fp;
	char buf[BUFSIZ];
	u64 val;

	if ((fp = fopen(_PROC_DISKDUMP, "r")) == NULL) {
		return failed;
	}

	while (fgets(buf, BUFSIZ, fp) != NULL) {
		if (strstr(buf, param_name) != NULL) {
			if (sscanf(buf, "# %*s %Lu", &val) != 1) {
				fclose(fp);
				return failed;
			}
			fclose(fp);
			return val;	/* Got an exact value. */
		}
	}

	fclose(fp);
	return failed;
}

static inline int get_window_width(void)
{
	int rc;
	struct winsize winsz;

	rc = ioctl(0, TIOCGWINSZ, &winsz);
	if (rc < 0 || winsz.ws_col == 0) {
		return DEFAULT_WINSZ;
	} else {
		return winsz.ws_col;
	}
}

static inline void print_bar(int symbol, int len, int max)
{
	int i;

	if (!(0x21 <= symbol && symbol < 0x7f)) {
		return;
	}

	for (i = 0; i < len; i++) {
		Stderr("%c", symbol);
	}

	Stderr("%*c", max - len + 1, ' ');
}

static void print_progress(const char *item, int symbol, off_t now, off_t max)
{
	double progress;
	int bar_len, bar_avl;
	char *prefix = NULL, *suffix = NULL;

	progress = (double)now * 100.0
		 / (double)max;

	if (progress > 100.0) {
		progress = 100.0;
	}

	prefix = g_strdup_printf("%s: ", item);
	suffix = g_strdup_printf("[%5.1f%%]", progress);

	bar_avl = get_window_width() - strlen(prefix) - strlen(suffix) - 1;
	bar_len = progress / 100 * bar_avl;

	Stderr(prefix);
	print_bar(symbol, bar_len, bar_avl);
	Stderr(suffix);
	Stderr("\r");
}

static inline void erase_bar(const char *item)
{
	int bar_avl;
	char *prefix = NULL, *suffix = NULL;

	prefix = g_strdup_printf("%s: ", item);
	suffix = g_strdup_printf("[%5.1f%%]", 100.0);

	bar_avl = get_window_width() - strlen(prefix);

	Stderr(prefix);
	Stderr("%*c", bar_avl, ' ');
	Stderr("\r");
	Stderr(prefix);
	Stderr(suffix);
}

void progress_bar(const char *item, int symbol, off_t now, off_t max)
{
	time_t tm;
	static time_t last_time = 0;

	tm = time(NULL);

	if (tm - last_time < 1) {
		return;
	}

	last_time = tm;

	print_progress(item, symbol, now, max);
}

void progress_bar_post(const char *item, int symbol, off_t now, off_t max)
{
	print_progress(item, symbol, now, max);
	erase_bar(item);
	Stderr("\n");
}

int execute_script(const char *script, const char *args)
{
	struct stat sbuf;
	char *filename;
	char *cmdline;
	int res = -1;

	filename = g_strconcat(script, NULL);

	if (stat(filename, &sbuf) == 0) {
		cmdline = g_strdup_printf("%s '%s'",
				filename, (args)? args : "");
		res = system(cmdline);
		g_free(cmdline);
	}

	g_free(filename);
	return res;
}

int rm_dir(const char *path)
{
	char *rm;
	int ret;

	rm = g_strdup_printf("rm -rf %s", path);

	ret = system(rm);

	g_free(rm);
	return ret;
}

char *convert_ip_addr(guint32 ip)
{
	char *ip_addr;

	ip_addr = g_strdup_printf("%d.%d.%d.%d",
			(ip >> 24) & 0xff,
			(ip >> 16) & 0xff,
			(ip >>  8) & 0xff,
			(ip      ) & 0xff);

	return ip_addr;
}

bool is_swapon(void)
{
	FILE *fp;
	char line[BUFSIZ];

	if ((fp = fopen("/proc/swaps", "r")) == NULL)
		return false;

	if (fgets(line, BUFSIZ, fp) == NULL) {
		fclose(fp);
		return false;
	}

	fclose(fp);
	return true;
}
